@@ -1,7 +1,7 @@
 __author__ = 'Altertech, http://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2018-2020 Altertech Group'
 __license__ = 'Apache License 2.0'
-__version__ = '2.0.41'
+__version__ = '2.0.42'
 
 from flask import request, jsonify, Response, abort, send_file, make_response
 from flask_restx import Api, Resource, reqparse
@@ -46,6 +46,24 @@ from pyaltt2.network import netacl_match
 
 success = {'ok': True}
 
+authenticators = []
+"""
+custom authenticator is a method defined as:
+
+    def myauth(ip, f, *args, **kwargs):
+        # ip - remote IP address
+        # f - requested API function
+        # *args, **kwargs - function arguments
+
+custom authenticator return values:
+
+* True: API request can be passed
+* None or False: ignored, go to next authenticator
+
+in case of unsuccessful authentication attempt, custom authenticator should
+call abort(status) or raise an exception
+"""
+
 
 def arr_from_str(s):
     if not isinstance(s, str) or s.find('|') == -1: return s
@@ -131,15 +149,7 @@ def public_method(f):
     return do
 
 
-def custom_authenticator(f, ip, *args, **kwargs):
-    """
-    can be overriden
-    """
-    logger.debug(f'API {ip} calling {f.__qualname__} {args} {kwargs}')
-    return False
-
-
-def master_authenticator(f, ip, *args, **kw):
+def master_authenticator(ip, f, *args, **kw):
     logger.debug(f'API admin call {ip} method: {f.__qualname__}, args: {args}, '
                  f'kwargs: {kw}')
     key = request.headers.get('X-Auth-Key', kw.get('k'))
@@ -165,11 +175,13 @@ def authenticated_method(f):
         ip = get_real_ip()
         payload = request.json if request.json else {}
         kw = {**kwargs, **payload}
-        if not custom_authenticator(f, ip, *args, **kw) and \
-                not master_authenticator(f, ip, *args, **kw):
-            abort(403)
+        for c in authenticators:
+            if c(ip, f, *args, **kw):
+                break
         else:
-            return f(*args, **kw)
+            if not master_authenticator(ip, f, *args, **kw):
+                abort(403)
+        return f(*args, **kw)
 
     return do
 
